function mOpt = oneLinearCvxDemod(s,fs,cvxValue)

%
% mOpt = oneLinearCvxDemod(s,fs,cvxValue)
%
% Function to run a linear-domain demodulation on a single window s of 
% sampling rate fs.  Returns estimated modulator mOpt.  Use linearCvxDemod
% to perform windowed demodulations of a full signal.
%
% Parameter cvxValue allows use of cvx if set to 1 (provided it is 
% available). 0 will not allow cvx to be used.
%
% Optimization Demodulation Package v1.0
% Gregory Sell 4/5/2010

scale = max(abs(s));
s = s/scale;

N = length(s);
dftMat = fft(eye(N))/sqrt(N);

binHz = fs/N*(0:N-1)';
midHz = 40;
weight = 100*(1+exp(.5*max(midHz-binHz,binHz-(fs-midHz)))).^(-1);
modMat = diag(weight)*dftMat;

save LinearWeight weight binHz

M = real((modMat')*modMat + eye(N));

if(exist('cvx_begin','file')==0 || cvxValue == 0)
    opt = optimset('Display','off');
    prob.H = M;
    prob.f = [];
    prob.Aineq = [];
    prob.bineq = [];
    prob.Aeq = [];
    prob.beq = [];
    prob.lb = abs(s);
    prob.ub = [];
    prob.x0 = ones(N,1);
    prob.options = opt;
    prob.solver = 'quadprog';
    mOpt = quadprog(prob);
else
    cvx_begin
    variable mOpt(N,1)
	minimize quad_form(mOpt,M);
    subject to
    mOpt >= abs(s);
	cvx_end
end

mOpt = mOpt*scale;