function mOpt = oneLogCvxDemod(sA,fs,w)

%
% mOpt = oneLogCvxDemod(s,fs)
% NOTE: Requires cvx
%
% Function to run a logarithmic-domain demodulation on a single window s of 
% sampling rate fs.  Returns estimated modulator mOpt.  Use logCvxDemod
% to perform windowed demodulations of a full signal.
%
% Optimization Demodulation Package v1.0
% Gregory Sell 4/5/2010

if(exist('cvx_begin','file')==0)
    disp('Error: Logarithmic Demodulation requires cvx (availabe at http://www.stanford.edu/~boyd/cvx)');
    mOpt = [];
    return;
end

N = length(sA);

sA(sA==0) = eps;

dftMat = fft(eye(N))/sqrt(N);
dftMatC = dftMat(2:end,:);

diffMat = (diag(ones(N,1),0) + diag(-1*ones(N-1,1),1))/2;

firstDiffMat = diffMat(2:N-1,:);

secondDiffMat = diffMat*diffMat;
secondDiffMat = secondDiffMat(1:N-2,:);

w = w.*[(fs/1000)^2 (fs/1000)^2 1 1];
cvx_begin
variable logM(N);
variables logCR(N) logCI(N);
minimize .5*w(1)*sum(secondDiffMat*logM + .5*(firstDiffMat*logM).^2) + w(2)*norm(secondDiffMat*logM,1) + w(3)*norm(logCR) + w(4)*norm(dftMatC*logCR,1)
subject to
abs(logM + 2*logCR - 2*real(log(sA))) <= (10^(-16));
2*logCI == 2*imag(log(sA));
cvx_end

mOpt = exp(logM/2);