function m = optimizeModSign(mPos)

%
% m = optimizeModSign(mPos)
%
% This function takes a positive modulator signal and finds the optimal
% combination of the modulator and its negative to minimize slope
% discontinuity.  The solution is returned as m.
%
% Note that the returned solution begins arbitrarily with a positive
% modulator, so, in cases where the modulator actually begins negative,
% -1*m should be used instead.
%

mNeg = -1*mPos;

N = length(mPos);
m = ones(N,1);
m(1) = mPos(1);
lastDiffs = zeros(3,1);

% Search set of solutions restricted to positive and negative Hilbert
% envelope to find the combination that minimizes jumps in the first
% derivative (with no lookahead).
for i=2:N
    posDiff = mPos(i) - m(i-1);
    negDiff = mNeg(i) - m(i-1);
    posDist = abs(mean(lastDiffs) - posDiff);
    negDist = abs(mean(lastDiffs) - negDiff);
    if (posDist > negDist)
        m(i) = mNeg(i);
        lastDiffs = circshift(lastDiffs,1);
        lastDiffs(1) = negDiff;
    else
        m(i) = mPos(i);
        lastDiffs = circshift(lastDiffs,1);
        lastDiffs(1) = posDiff;
    end
end
